package cn.org.bjca.trust.android.lib.im.im.manager;

import android.annotation.SuppressLint;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;

import androidx.annotation.NonNull;

import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallbackExtended;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cn.org.bjca.trust.android.lib.im.cfg.Constant;
import cn.org.bjca.trust.android.lib.im.common.DeviceHelper;
import cn.org.bjca.trust.android.lib.im.common.ToolsHelper;
import cn.org.bjca.trust.android.lib.im.common.json.GsonImplHelp;
import cn.org.bjca.trust.android.lib.im.enums.MsgType;
import cn.org.bjca.trust.android.lib.im.enums.PacketType;
import cn.org.bjca.trust.android.lib.im.im.bean.PacketMessage;
import cn.org.bjca.trust.android.lib.im.im.kit.RequestImCallback;
import cn.org.bjca.trust.android.lib.im.im.msg.ConnectMessage;
import cn.org.bjca.trust.android.lib.im.im.msg.ConnectedMessage;
import cn.org.bjca.trust.android.lib.im.im.msg.ConversationMessage;
import cn.org.bjca.trust.android.lib.im.im.msg.RequestMessage;
import cn.org.bjca.trust.android.lib.im.im.msg.ResultMessage;
import cn.org.bjca.trust.android.lib.im.im.msg.SendAckMessage;
import cn.org.bjca.trust.android.lib.im.im.msg.message.SZYXMessage;
import cn.org.bjca.trust.android.lib.im.im.msg.message.SZYXTextMessage;
import cn.org.bjca.trust.android.lib.im.im.opt.ImConnectOptions;
import cn.org.bjca.trust.android.lib.im.kit.IMInterface;
import cn.org.bjca.trust.android.lib.im.kit.IMSDKCallback;
import cn.org.bjca.trust.android.lib.im.kit.IMSDKListener;
import cn.org.bjca.trust.android.lib.im.kit.MsgListener;
import kotlin.text.Charsets;

public class ImManager implements IMInterface {
    private final String TAG = "ImManager";

    //登录成功
    private String topic_connected;

    // 新消息到达
    private String topic_arrive;
    private String topic_conversation;

    // 发送消息应答
    private String topic_send_ack;

    // 心跳应答
    private String topic_pang;


    // 数据请求回复
    private String topic_result;

    // 系统消息
    private String topic_system;

    private MqttClient mqttClient;
    private MqttConnectOptions connectOptions;

    private int status = -1; // -1未连接|0连接成功|1连接中

    private IMSDKListener statusListener;

    private final List<MsgListener> msgListeners = new ArrayList<>();
    private final Map<String, IMSDKCallback> msgListenerMap = new HashMap<>();
    private final Map<String, RequestImCallback> requestMap = new HashMap<>();


    @Override
    public void addMsgListener(MsgListener listener) {
        msgListeners.add(listener);
    }

    @Override
    public void removeMsgListener(MsgListener listener) {
        msgListeners.remove(listener);
    }

    @Override
    public void setStatusListener(IMSDKListener listener) {
        this.statusListener = listener;
    }

    @Override
    public void removeStatusListener(IMSDKListener listener) {
        this.statusListener = null;
    }


    private void initTopic() {
        topic_send_ack = Constant.getSdkAppID() + "/" + PacketType.SEND_ACK + Constant.getUserId();
        topic_connected = Constant.getSdkAppID() + "/" + PacketType.CONNECTED + Constant.getUserId();
        topic_arrive = Constant.getSdkAppID() + "/" + PacketType.SEND + Constant.getUserId();
        topic_conversation = Constant.getSdkAppID() + "/" + PacketType.CONVERSATION + Constant.getUserId();
        topic_pang = Constant.getSdkAppID() + "/" + PacketType.PANG + Constant.getUserId();
        topic_result = Constant.getSdkAppID() + "/" + PacketType.RESULT + Constant.getUserId();
        topic_system = Constant.getSdkAppID() + "/" + PacketType.SYSTEM + Constant.getUserId();
    }

    @Override
    public void login(ImConnectOptions imConnectOptions) {
        if (this.isConnect()) this.logout();

        if (null == connectOptions) connectOptions = new MqttConnectOptions();
        connectOptions.setCleanSession(false);
        connectOptions.setUserName(imConnectOptions.getToken());
        connectOptions.setPassword(imConnectOptions.getPassword().toCharArray());
        connectOptions.setConnectionTimeout(30);
        connectOptions.setKeepAliveInterval(20);
        connectOptions.setAutomaticReconnect(true);
        initTopic();

        try {
            mqttClient = new MqttClient("tcp://" + imConnectOptions.getHost() + ":" + imConnectOptions.getPort(),
                    Constant.getSdkAppID() + imConnectOptions.getToken() + System.currentTimeMillis(), new MemoryPersistence());
            mqttClient.setCallback(new MqttCallbackExtended() {
                @Override
                public void connectComplete(boolean reconnect, String serverURI) {
                    Log.e("======>connectComplete", reconnect + "::" + serverURI);
                    status = 0;
                    if (reconnect) return;
                    try {
                        mqttClient.subscribe(topic_connected, 2);
                        mqttClient.subscribe(topic_arrive, 2);
                        mqttClient.subscribe(topic_send_ack, 2);
                        mqttClient.subscribe(topic_pang, 2);
                        mqttClient.subscribe(topic_result, 2);
                        mqttClient.subscribe(topic_system, 2);
                        mqttClient.subscribe(topic_conversation, 2);
                        Log.e("======>connectComplete", topic_send_ack);
                    } catch (MqttException e) {
                        Log.e(TAG, "=====>connectComplete", e);
                    }
                    // 发送登录消息
                    sendConnectMsg();
                }

                @Override
                public void connectionLost(Throwable cause) {
                    Log.e("======>connectionLost", "", cause);
                    status = -1;
                    if (null != statusListener) statusListener.onConnectionLost();
                }

                @Override
                public void messageArrived(String topic, MqttMessage message) throws Exception {
                    String msg = new String(message.getPayload(), Charsets.UTF_8);
                    // 登录应答
                    if (topic.equals(topic_connected)) {
                        ConnectedMessage connectedMessage = GsonImplHelp.get().toObject(msg, ConnectedMessage.class);
                        if (connectedMessage.getStatus() == 200) {
                            if (statusListener != null) statusListener.onConnectSuccess();
                        } else {
                            if (statusListener != null)
                                statusListener.onConnectFailed(connectedMessage.getStatus(), connectedMessage.getMsg());
                        }
                    }
                    // 发送消息应答
                    else if (topic.equals(topic_send_ack)) {
                        SendAckMessage sendAckMessage = GsonImplHelp.get().toObject(msg, SendAckMessage.class);

                        Message m = new Message();
                        m.obj = sendAckMessage;
                        m.what = 10002;
                        handler.sendMessage(m);

                    }
                    // 接口请求应答
                    else if (topic.equals(topic_result)) {
                        ResultMessage resultMessage = GsonImplHelp.get().toObject(msg, ResultMessage.class);

                        Message m = new Message();
                        m.obj = resultMessage;
                        m.what = 10004;
                        handler.sendMessage(m);

                    }
                    // 会话更新
                    else if (topic.equals(topic_conversation)) {
                        ConversationMessage cm = GsonImplHelp.get().toObject(msg, ConversationMessage.class);
                        Log.d("======>会话列表更新", topic + "::\n" + cm);
                    }
                    // 新消息送达
                    else if (topic.equals(topic_arrive)) {
                        SZYXMessage szyxMessage = GsonImplHelp.get().toObject(msg, SZYXMessage.class);
                        sendAckMsg(szyxMessage.getMsgId());
                        Log.d("======>新消息送达", topic + "::\n" + msg);
                        for (MsgListener listener : msgListeners) {
                            listener.messageArrive(new ArrayList<SZYXMessage>() {
                                {
                                    add(szyxMessage);
                                }
                            });
                        }
                    } else
                        Log.e("======>未知消息", topic + "::" + msg);


                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken token) {
                    try {
                        if (token.getMessage() != null && token.getMessage().getPayload() != null)
                            Log.e("======>deliveryComplete", new String(token.getMessage().getPayload(), Charsets.UTF_8));
                    } catch (MqttException ignored) {
                    }
                }

            });
        } catch (MqttException e) {
            Log.e(TAG, "login: ", e);
        }

        connect();
    }

    private void connect() {
        this.status = 1;
        new Thread(() -> {
            try {
                if (!mqttClient.isConnected()) {
                    mqttClient.connect(connectOptions);
                }

            } catch (MqttException e) {
                Log.e(TAG, "connect: ", e);
                if (statusListener != null) statusListener.onConnectFailed(3001, e.getMessage());
            }
        }
        ).start();

    }

    @Override
    public void logout() {

    }

    @Override
    public boolean isConnect() {
        return this.mqttClient != null && this.mqttClient.isConnected();
    }

    @Override
    public boolean isConnecting() {
        return this.status == 1;
    }

    @Override
    public SZYXMessage createMsg(MsgType msgType, String toUserId, boolean isGroup, Object message, String describe) {

        SZYXMessage szyxMessage = new SZYXMessage(msgType);
        szyxMessage.setArrive(false);
        szyxMessage.setDescribe(describe);
        szyxMessage.setGroup(isGroup);
        szyxMessage.setStatus(1);
        szyxMessage.setFromClientId(Constant.getUserId());
        if (isGroup)
            szyxMessage.setGroupID(toUserId);
        else
            szyxMessage.setToClientId(toUserId);
        switch (msgType) {
            case Text:
                szyxMessage.setTextMessage((SZYXTextMessage) message);
                break;
            case Voice:
                break;
        }

        return szyxMessage;
    }

    @Override
    public void sendSZYXMessage(SZYXMessage message, IMSDKCallback callback) {
        if (ToolsHelper.isNull(message.getToClientId()) && ToolsHelper.isNull(message.getGroupID())) {
            if (null != callback) callback.failed(202, "没有消息接收者[toClientId]");
            for (MsgListener listener : msgListeners) {
                listener.statusChange(message.getMsgId(), -1);
            }
        } else
            sendPacketMessage(PacketType.SEND, message, callback);
    }

    @Override
    public void request(RequestMessage message, RequestImCallback callback) {
        this.sendPacketMessage(PacketType.REQUEST, message, new IMSDKCallback() {
            @Override
            public void success() {

            }

            @Override
            public void failed(int code, String error) {
                callback.failed(4001, error);
            }
        });

        requestMap.put(message.getMsgId(), callback);
        Message m = new Message();
        m.obj = message.getMsgId();
        m.what = 10003;
        handler.sendMessageDelayed(m, 600);
    }

    private void sendConnectMsg() {
        ConnectMessage message = new ConnectMessage(DeviceHelper.getDevice());
        sendPacketMessage(PacketType.CONNECT, message, null);
    }

    private void sendAckMsg(String msgId) {
        SendAckMessage message = new SendAckMessage();
        message.setAckId(msgId);
        sendPacketMessage(PacketType.SEND_ACK, message, null);
    }

    private void sendPacketMessage(PacketType packetType, PacketMessage message, IMSDKCallback callback) {
        if (null == connectOptions) {
            if (null != callback) callback.failed(204, "未登录");
            return;
        }
        if (!isConnect() || isConnecting()) {
            if (null != callback) callback.failed(205, "im未连接");
            return;
        }
        String msg = GsonImplHelp.get().toJson(message);
        try {
            mqttClient.publish("server" + packetType, msg.getBytes(Charsets.UTF_8), 2, false);
            // 如果是发送的消息，需要应答
            if (PacketType.SEND == packetType) {
                if (null != callback)
                    msgListenerMap.put(message.getMsgId(), callback);
                else msgListenerMap.put(message.getMsgId(), new IMSDKCallback() {
                    @Override
                    public void success() {

                    }

                    @Override
                    public void failed(int code, String error) {

                    }
                });
                Message m = new Message();
                m.obj = message.getMsgId();
                m.what = 10001;
                handler.sendMessageDelayed(m, 1500);
            }
        } catch (MqttException e) {
            if (null != callback) callback.failed(206, e.getMessage());
        }

    }

    private final Handler handler = new Handler(Looper.getMainLooper()) {
        @SuppressLint("HandlerLeak")
        @Override
        public void handleMessage(@NonNull Message msg) {
            super.handleMessage(msg);
            switch (msg.what) {
                case 10001:
                    String msgId = (String) msg.obj;
                    IMSDKCallback callback = msgListenerMap.get(msgId);
                    if (null != callback) {
                        callback.failed(207, "发送超时");
                        msgListenerMap.remove(msgId);

                        for (MsgListener listener : msgListeners) {
                            listener.statusChange(msgId, -1);
                        }
                    }
                    break;
                case 10002:
                    SendAckMessage sendAckMessage = (SendAckMessage) msg.obj;
                    String msgId1 = sendAckMessage.getAckId();
                    IMSDKCallback callback1 = msgListenerMap.get(msgId1);
                    if (null != callback1) {
                        for (MsgListener listener : msgListeners) {
                            if (sendAckMessage.getAckStatus() == 0)
                                listener.statusChange(msgId1, 0);
                            else listener.statusChange(msgId1, -1);
                        }
                        if (sendAckMessage.getAckStatus() == 0) callback1.success();
                        else callback1.failed(208, "服务端转发失败");
                        msgListenerMap.remove(msgId1);
                    }
                    break;
                case 10003:
                    String msgId3 = (String) msg.obj;
                    RequestImCallback imCallback = requestMap.get(msgId3);
                    if (null != imCallback) {
                        imCallback.failed(4001, "发送超时");
                        requestMap.remove(msgId3);
                    }
                    break;
                case 10004:
                    ResultMessage resultMessage = (ResultMessage) msg.obj;
                    String msgId4 = resultMessage.getAckId();
                    RequestImCallback callback4 = requestMap.get(msgId4);
                    if (null != callback4) {
                        callback4.success(resultMessage);
                        requestMap.remove(msgId4);
                    }
                    break;
            }
        }
    };

}
