package cn.org.bjca.trust.android.lib.im.manager;

import android.content.Context;

import cn.org.bjca.trust.android.lib.im.BuildConfig;
import cn.org.bjca.trust.android.lib.im.SZYXDbHelper;
import cn.org.bjca.trust.android.lib.im.bean.UserInfo;
import cn.org.bjca.trust.android.lib.im.cfg.Constant;
import cn.org.bjca.trust.android.lib.im.common.DeviceHelper;
import cn.org.bjca.trust.android.lib.im.http.HttpManage;
import cn.org.bjca.trust.android.lib.im.im.IMHelper;
import cn.org.bjca.trust.android.lib.im.im.manager.UserManager;
import cn.org.bjca.trust.android.lib.im.im.opt.ImConnectOptions;
import cn.org.bjca.trust.android.lib.im.kit.IMSDKCallback;
import cn.org.bjca.trust.android.lib.im.kit.IMSDKListener;
import cn.org.bjca.trust.android.lib.im.kit.SdkInterface;
import cn.org.bjca.trust.android.lib.im.repository.Service;
import cn.org.bjca.trust.android.lib.im.repository.bean.LoginBean;
import cn.org.bjca.trust.android.lib.im.repository.data.LoginData;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;

public class SZYXImManager implements SdkInterface {

    private IMSDKListener imsdkListener;

    @Override
    public void setIMSDKListener(IMSDKListener listener) {
        this.imsdkListener = listener;
        IMHelper.getInstance().setStatusListener(listener);
    }

    @Override
    public void removeIMSDKListener(IMSDKListener listener) {
        IMHelper.getInstance().removeStatusListener(listener);
        this.imsdkListener = null;
    }

    @Override
    public void init(Context context, String sdkAppID, IMSDKCallback callback) {
        Constant.setSdkAppID(sdkAppID);
        SZYXDbHelper.get(context);
        DeviceHelper.getDevice();
    }

    @Override
    public void login(String userID, String userSig, IMSDKCallback callback) {
        if (null == Constant.getSdkAppID()) {
            if (null != callback) callback.failed(203, "未初始化");
        }
        Constant.setUserId(userID);
        Disposable d = HttpManage.getApi(Service.class)
                .register(new LoginData(userID, userSig))
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(httpResult -> {
                    if (httpResult.getCode() == 200) {
                        if (null != callback) callback.success();
                        if (null != imsdkListener) imsdkListener.onConnecting();
                        LoginBean bean = httpResult.getData();
                        UserManager.getInstance().setUserInfo(bean.getUserInfo());
                        Constant.setToken(bean.getUserInfo().getToken());
                        imLogin(bean.getHost(), bean.getPort(), bean.getUserInfo().getToken(), bean.getUserInfo().getPassword());
                    } else if (null != callback) callback.failed(1001, httpResult.getMsg());
                }, throwable -> {
                    if (null != callback) callback.failed(1001, throwable.getMessage());
                });
    }

    private void imLogin(String host, String port, String token, String password) {
        IMHelper.getInstance().login(new ImConnectOptions(host, port, token, password));
    }

    @Override
    public void logout(IMSDKCallback callback) {
        IMHelper.getInstance().logout();
    }

    @Override
    public UserInfo getUser() {
        return UserManager.getInstance().getUserInfoForMsg();
    }
    @Override
    public UserInfo getUser(String userId) {
        return UserManager.getInstance().getUserInfoForMsg();
    }


    @Override
    public String getVersion() {
        return BuildConfig.versionName;
    }
}
