package cn.org.bjca.trust.android.lib.im.request;

import java.lang.reflect.Type;
import java.util.List;

import cn.org.bjca.trust.android.lib.im.common.json.GsonImplHelp;
import cn.org.bjca.trust.android.lib.im.common.json.reflect.TypeBuilder;
import cn.org.bjca.trust.android.lib.im.http.HttpResult;
import cn.org.bjca.trust.android.lib.im.im.IMHelper;
import cn.org.bjca.trust.android.lib.im.im.kit.RequestImCallback;
import cn.org.bjca.trust.android.lib.im.im.msg.RequestMessage;
import cn.org.bjca.trust.android.lib.im.im.msg.ResultMessage;

public class RequestManager implements RequestInterface {


    @Override
    public <T> void request(String path, Object data, Class<T> clazz, RequestCallback<T> callback) {

        RequestMessage rm = new RequestMessage(path, GsonImplHelp.get().toJson(data));

        IMHelper.getInstance().request(rm, new RequestImCallback() {

            @Override
            public void success(ResultMessage message) {
                Type type = TypeBuilder
                        .newInstance(HttpResult.class)
                        .addTypeParam(clazz)
                        .build();
                HttpResult<T> h = GsonImplHelp.get().toObject(message.getBody(), type);
                if (h.getCode() == 200) {
                    callback.success(h);
                } else {
                    callback.failed(h.getCode(), h.getMsg());
                }
            }

            @Override
            public void failed(int code, String error) {
                if (null != callback) {
                    callback.standby(new RequestCallback<T>() {
                        @Override
                        public void success(HttpResult<T> result) {
                            callback.success(result);
                        }

                        @Override
                        public void failed(int code, String error) {
                            callback.failed(code, error);
                        }

                        @Override
                        public void standby(RequestCallback<T> imCallback) {

                        }
                    });

                }
            }

        });
    }

    @Override
    public <T> void requestList(String path, Object data, Class<T> clazz, RequestCallback<List<T>> callback) {

        RequestMessage rm = new RequestMessage(path, GsonImplHelp.get().toJson(data));

        IMHelper.getInstance().request(rm, new RequestImCallback() {

            @Override
            public void success(ResultMessage message) {
                Type type = TypeBuilder
                        .newInstance(HttpResult.class)
                        .beginSubType(List.class)
                        .addTypeParam(clazz)
                        .endSubType()
                        .build();
                HttpResult<List<T>> h = GsonImplHelp.get().toObject(message.getBody(), type);
                if (h.getCode() == 200) {
                    callback.success(h);
                } else {
                    callback.failed(h.getCode(), h.getMsg());
                }
            }

            @Override
            public void failed(int code, String error) {
                if (null != callback) {
                    callback.standby(new RequestCallback<List<T>>() {
                        @Override
                        public void success(HttpResult<List<T>> result) {
                            callback.success(result);
                        }

                        @Override
                        public void failed(int code, String error) {
                            callback.failed(code, error);
                        }

                        @Override
                        public void standby(RequestCallback<List<T>> imCallback) {

                        }
                    });

                }
            }

        });
    }
}
