package cn.org.bjca.trust.android.imdemo.ui;

import android.os.Bundle;
import android.view.View;

import androidx.lifecycle.ViewModelProvider;
import androidx.recyclerview.widget.LinearLayoutManager;

import com.xuqm.base.common.ToolsHelper;
import com.xuqm.base.ui.BaseActivity;
import com.xuqm.base.view.enu.Status;

import cn.org.bjca.trust.android.imdemo.R;
import cn.org.bjca.trust.android.imdemo.databinding.ActivityChatBinding;
import cn.org.bjca.trust.android.imdemo.ui.adapter.ChatAdapter;
import cn.org.bjca.trust.android.imdemo.vm.ChatVM;
import cn.org.bjca.trust.android.lib.im.SZYXImSdk;
import cn.org.bjca.trust.android.lib.im.kit.IMSDKCallback;

public class ChatActivity extends BaseActivity<ActivityChatBinding> {

    private ChatVM vm;
    private ChatAdapter adapter;

    @Override
    public int getLayoutId() {
        return R.layout.activity_chat;
    }

    @Override
    public void initView(Bundle savedInstanceState) {
        super.initView(savedInstanceState);
        this.adapter = new ChatAdapter(mContext);
        vm = new ViewModelProvider(this).get(ChatVM.class);
        String userId = getIntent().getStringExtra("userId");
        String userName = getIntent().getStringExtra("userName");
        boolean isGroup = getIntent().getBooleanExtra("isGroup", false);
        setTitleText(userName + "(" + (isGroup ? "群组" : userId) + ")");
        getBaseBinding().baseToolbar.getToolbarMenu().setVisibility(View.VISIBLE);

        getBinding().baseRecyclerView.setAdapter(adapter);
        getBinding().baseRecyclerView.setLayoutManager(new LinearLayoutManager(this));

        getBinding().refreshLayout.setOnRefreshListener(() -> vm.onRefresh());

        getBinding().btnSend.setOnClickListener(v -> {
            String msg = getBinding().et.getText().toString();
            if (!ToolsHelper.isNull(msg)) {
                vm.add(SZYXImSdk.getMessageManager().sendMsgForText(userId, isGroup, msg, new IMSDKCallback() {
                    @Override
                    public void success() {
                        ToolsHelper.showMessage("发送消息成功");
                    }

                    @Override
                    public void failed(int code, String error) {
                        ToolsHelper.showMessage("发送消息失败" + error);
                    }
                }));
            } else ToolsHelper.showMessage("请输入内容后再发送");
        });

        getBinding().baseEmptyView.setStatus(Status.NO_DATA);

        vm.getMessage().observe(this, messages -> {
            getBinding().refreshLayout.setRefreshing(false);
            if (null == messages || messages.size() == 0) {
            } else {
                getBinding().baseEmptyView.setStatus(Status.DISMISS);
                adapter.setmDatas(messages);
            }
        });
        vm.getMessageList(userId);
    }
}